#
# comp-house-fishing.R, 6 Apr 24
#
# Data from:
#
# Companies house data
# https://download.companieshouse.gov.uk/en_output.html
# Postcode data https://www.freemaptools.com/download-uk-postcode-lat-lng.htm

library("colorspace")
library("plyr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(2)

# Count number of software companies
cnt_company=function(df)
{
if (!df$known_outcode)
   return(NULL)

return(data.frame(all_comp=nrow(df), soft_comp=sum(df$is_soft_comp)))
}


# Download this data from: https://download.companieshouse.gov.uk/en_output.html
# xz file is 241.8 MB
ch=read.csv("../company-data/BasicCompanyDataAsOneFile-2024-04-01.csv.xz", as.is=TRUE)

outcode=read.csv("postcode-outcodes.csv", as.is=TRUE) # postcode outcode lat/long

ch$DateInc=as.Date(ch$IncorporationDate, format="%d/%m/%Y")
ch$outcode=toupper(substr(ch$RegAddress.PostCode, 1, regexpr(" ", ch$RegAddress.PostCode)-1))
ch$SIC1=as.numeric(substr(ch$SICCode.SicText_1, 1, regexpr(" ", ch$SICCode.SicText_1)-1))
# ch$SIC2=as.numeric(substr(ch$SICCode.SicText_2, 1, regexpr(" ", ch$SICCode.SicText_2)-1))

sic_rank=count(ch$SIC1)
sic_rank=sic_rank[order(sic_rank$freq, decreasing=TRUE), ]

soft_SIC=c(62011, # Ready-made interactive leisure and entertainment software development
		62012, # Business and domestic software development
		62020, # Information technology consultancy activities
		62030, # Computer facilities management activities
		62090, # Other information technology service activities
		63110, # Data processing, hosting and related activities
		63120) # Web portals

ch$known_outcode=(ch$outcode %in% outcode$postcode)
ch$is_soft_comp=(ch$SIC1 %in% soft_SIC)

soft_comp=subset(ch, is_soft_comp)

#  table(soft_comp$SIC1)
# 
#  62011  62012  62020  62030  62090  63110  63120 
#   7217  68834 134461   3457  57132   7839   8225

#  table(soft_comp$SIC1)/nrow(ch)

# table(soft_comp$CompanyCategory)
# 
#                                                                Community Interest Company 
#                                                                                       477 
# PRI/LBG/NSC (Private, Limited by guarantee, no share capital, use of 'Limited' exemption) 
#                                                                                       269 
#                     PRI/LTD BY GUAR/NSC (Private, limited by guarantee, no share capital) 
#                                                                                       709 
#                                                                   Private Limited Company 
#                                                                                    285525 
#                                                                         Private Unlimited 
#                                                                                         1 
#                                                                 Private Unlimited Company 
#                                                                                        28 
#                                                                    Public Limited Company 
#                                                                                       156 
# table(soft_comp$CompanyStatus)

Date_today=as.Date("07/04/2024", format="%d/%m/%Y")

# table(ch$CompanyCategory)

length(unique(ch$SICCode.SicText_1))
# [1] 1168
length(unique(ch$SICCode.SicText_2))
# [1] 1001
length(unique(ch$SICCode.SicText_3))
# [1] 867

act_year_cnt=count(round(as.numeric(Date_today-ch$DateInc)/365.25, digits=1))

plot(act_year_cnt, log="xy",
	xlim=c(0.1, 200),
	xlab="Years live", ylab="Companies")

# mysql/mariadb conflicts on Ubuntu prevented me from installing gdal
# So I went with simpler map drawing
library("maps")
library("mapdata")

uk_outline=map("worldHires", "UK", xlim=c(-8.2, 2), ylim=c(49.5, 59.5), resolution=0.4, mar=c(0.5, 0.5, 0.5, 0.5))

oc_company=ddply(ch, .(outcode), cnt_company)

oc=merge(outcode, oc_company, by.x="postcode", by.y="outcode")

av_soft_comp=sum(oc$soft_comp)/sum(oc$all_comp)

oc$ss_heat=(oc$soft_comp/oc$all_comp)/av_soft_comp

oc=oc[order(oc$ss_heat, decreasing=TRUE), ]

clean_oc=subset(oc, soft_comp > 10) # require at least 10 software companies

clean_oc$ss_heat=round(10*clean_oc$ss_heat, digits=0)
# heat_col=rev(rainbow(max(clean_oc$ss_heat)))
heat_col=rev(rainbow(40))  # one outcode is not covered, but results look better

points(clean_oc$longitude, clean_oc$latitude, pch=".", cex=3, col=heat_col[clean_oc$ss_heat])

dev.copy(dev=png, file="cc-gr-nat-average.png")
dev.off()

# Order by number of software companies
top_cc=clean_oc[order(clean_oc$soft_comp, decreasing=TRUE), ]

head(top_cc, n=20)

